@@ -17,23 +17,6 @@
  */
 
 #include <doclone/Image.h>
-#include <doclone/Clone.h>
-#include <doclone/Logger.h>
-#include <doclone/Operation.h>
-#include <doclone/DataTransfer.h>
-#include <doclone/DlFactory.h>
-
-#include <doclone/exception/Exception.h>
-#include <doclone/exception/ErrorException.h>
-#include <doclone/exception/WarningException.h>
-#include <doclone/exception/InvalidImageException.h>
-#include <doclone/exception/WrongImageTypeException.h>
-#include <doclone/exception/NoFsToolFoundException.h>
-#include <doclone/exception/NoUuidSupportException.h>
-#include <doclone/exception/NoLabelSupportException.h>
-#include <doclone/exception/NoMountSupportException.h>
-#include <doclone/exception/NoFitInDeviceException.h>
-#include <doclone/exception/TooMuchPartitionsException.h>
 
 #include <stdint.h>
 #include <sys/types.h>
@@ -44,6 +27,26 @@
 #include <sstream>
 #include <string>
 
+#include <config.h>
+
+#include <doclone/Clone.h>
+#include <doclone/Logger.h>
+#include <doclone/Operation.h>
+#include <doclone/DataTransfer.h>
+#include <doclone/DlFactory.h>
+#include <doclone/exception/Exception.h>
+#include <doclone/exception/ErrorException.h>
+#include <doclone/exception/WarningException.h>
+#include <doclone/exception/InvalidImageException.h>
+#include <doclone/exception/WrongImageTypeException.h>
+#include <doclone/exception/NoFsToolFoundException.h>
+#include <doclone/exception/NoUuidSupportException.h>
+#include <doclone/exception/NoLabelSupportException.h>
+#include <doclone/exception/NoMountSupportException.h>
+#include <doclone/exception/NoSelinuxSupportException.h>
+#include <doclone/exception/NoFitInDeviceException.h>
+#include <doclone/exception/TooMuchPartitionsException.h>
+
 namespace Doclone {
 
 /**
@@ -68,6 +71,7 @@ Image::~Image() {
 	}
 
 	this->_partitions.clear();
+	this->_partsInfo.clear();
 }
 
 /**
@@ -503,6 +507,11 @@ bool Image::canCreateCheck() const throw(Exception) {
 
 	bool retValue = true;
 
+#ifndef HAVE_SELINUX
+	NoSelinuxSupportException selinuxEx;
+	selinuxEx.logMsg();
+#endif
+
 	try {
 		// For each partition
 		for(unsigned int i = 0;i<this->_partitions.size()
@@ -563,6 +572,11 @@ bool Image::canRestoreCheck(const std::string &device, uint64_t size) const thro
 
 	bool retValue = true;
 
+#ifndef HAVE_SELINUX
+	NoSelinuxSupportException selinuxEx;
+	selinuxEx.logMsg();
+#endif
+
 	try {
 		switch(this->_type) {
 			case Doclone::IMAGE_DISK: {
