@@ -1,6 +1,6 @@
 /*
  *  libdoclone - library for cloning GNU/Linux systems
- *  Copyright (C) 2013 Joan Lledó <joanlluislledo@gmail.com>
+ *  Copyright (C) 2013, 2014 Joan Lledó <joanlluislledo@gmail.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,22 +17,6 @@
  */
 
 #include <doclone/Partition.h>
-#include <doclone/Clone.h>
-#include <doclone/Logger.h>
-#include <doclone/PartedDevice.h>
-#include <doclone/FsFactory.h>
-#include <doclone/Util.h>
-
-#include <doclone/exception/CancelException.h>
-#include <doclone/exception/ReadDataException.h>
-#include <doclone/exception/WriteDataException.h>
-#include <doclone/exception/ReceiveDataException.h>
-#include <doclone/exception/SendDataException.h>
-#include <doclone/exception/FormatException.h>
-#include <doclone/exception/MountException.h>
-#include <doclone/exception/UmountException.h>
-#include <doclone/exception/InvalidImageException.h>
-#include <doclone/exception/FileNotFoundException.h>
 
 #include <stdlib.h>
 #include <sys/mount.h>
@@ -43,10 +27,25 @@
 #include <sys/stat.h>
 #include <unistd.h>
 
-#include <glibmm-2.4/glibmm.h>
+#include <string>
+
 #include <blkid/blkid.h>
 
-#include <string>
+#include <doclone/Clone.h>
+#include <doclone/Logger.h>
+#include <doclone/PartedDevice.h>
+#include <doclone/FsFactory.h>
+#include <doclone/Util.h>
+#include <doclone/exception/CancelException.h>
+#include <doclone/exception/ReadDataException.h>
+#include <doclone/exception/WriteDataException.h>
+#include <doclone/exception/ReceiveDataException.h>
+#include <doclone/exception/SendDataException.h>
+#include <doclone/exception/FormatException.h>
+#include <doclone/exception/MountException.h>
+#include <doclone/exception/UmountException.h>
+#include <doclone/exception/InvalidImageException.h>
+#include <doclone/exception/FileNotFoundException.h>
 
 namespace Doclone {
 
@@ -234,6 +233,7 @@ void Partition::initFS() throw(Exception) {
 		}
 		blkid_tag_iterate_end(iter);
 	}
+	blkid_put_cache(cache);
 	
 	this->_fs = FsFactory::createFilesystem(info);
 
@@ -399,7 +399,7 @@ void Partition::initUUID() throw(Exception) {
 /**
  * \brief Calculates the used space in partition
  *
- * \return used space in bytes
+ * \return Used space in bytes
  */
 uint64_t Partition::usedSpace() throw(Exception) {
 	Logger *log = Logger::getInstance();
@@ -425,7 +425,7 @@ uint64_t Partition::usedSpace() throw(Exception) {
 	
 	this->doUmount();
 	
-	uint64_t retValue = used_blocks * info.f_bsize;
+	uint64_t retValue = used_blocks * info.f_frsize;
 	
 	log->debug("Partition::usedSpace(retValue=>%d) end", retValue);
 	return retValue;
@@ -481,7 +481,8 @@ void Partition::externalMount() throw(Exception) {
 	.append(">/dev/null 2>&1");
 			 
 	int exitValue;
-	Glib::spawn_command_line_sync( "sh -c '" + cmdline + "'", 0, 0, &exitValue );
+	Util::spawn_command_line_sync(cmdline, &exitValue, 0);
+
 	if (exitValue<0) {
 		MountException ex(this->_path);
 		throw ex;
@@ -623,7 +624,8 @@ void Partition::format() const throw(Exception) {
 		" "+this->_path.c_str()+" >/dev/null 2>&1";
 
 	int exitValue;
-	Glib::spawn_command_line_sync( "sh -c '" + cmdline + "'", 0, 0, &exitValue );
+	Util::spawn_command_line_sync(cmdline, &exitValue, 0);
+
 	if (exitValue<0) {
 		FormatException ex;
 		throw ex;
