@@ -17,10 +17,29 @@
  */
 
 #include <doclone/UnixFS.h>
+
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <unistd.h>
+#include <dirent.h>
+#include <string.h>
+#include <stdio.h>
+#include <utime.h>
+#include <errno.h>
+#include <endian.h>
+
+#include <string>
+#include <map>
+
+#include <config.h>
+#ifdef HAVE_SELINUX
+#include <selinux/selinux.h>
+#endif
+
 #include <doclone/Logger.h>
 #include <doclone/DataTransfer.h>
 #include <doclone/Util.h>
-
 #include <doclone/exception/Exception.h>
 #include <doclone/exception/FileNotFoundException.h>
 #include <doclone/exception/OpenFileException.h>
@@ -31,21 +50,6 @@
 #include <doclone/exception/ReadErrorsInDirectoryException.h>
 #include <doclone/exception/RestoreImageException.h>
 
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <unistd.h>
-#include <dirent.h>
-#include <selinux/selinux.h>
-#include <string.h>
-#include <stdio.h>
-#include <utime.h>
-#include <errno.h>
-#include <endian.h>
-
-#include <string>
-#include <map>
-
 namespace Doclone {
 
 /**
@@ -55,6 +59,13 @@ UnixFS::UnixFS()
 	: _hardLinks() {
 }
 
+/**
+ * \brief Clears the map of hard links
+ */
+UnixFS::~UnixFS() {
+	this->_hardLinks.clear();
+}
+
 /**
  * \brief Reads the data and metadata of a regular file in a unix filesystem and
  * transfers it to the destination.
@@ -69,7 +80,6 @@ void UnixFS::readRegularFile(const std::string &path, const struct stat &filesta
 	log->loopDebug("UnixFS::readRegularFile(path=>%s, filestat=>0x%x) start", path.c_str(), &filestat);
 	
 	Doclone::unixFile file = {};
-	security_context_t context;	// Selinux context of this file
 	DataTransfer *trns = DataTransfer::getInstance();
 
 	std::string fileName = basename(path.c_str());
@@ -81,13 +91,18 @@ void UnixFS::readRegularFile(const std::string &path, const struct stat &filesta
 	file.atime = filestat.st_atime;
 	file.mtime = filestat.st_mtime;
 
+#ifdef HAVE_SELINUX
 	// Selinux extended attributes
+	security_context_t context;	// Selinux context of this file
 	if(getfilecon (path.c_str(), &context)<0) {
 		strncpy(reinterpret_cast<char*>(file.se_xattr), "", 0);
 	}
 	else {
 		strncpy (reinterpret_cast<char*>(file.se_xattr), context, 128);
 	}
+#else
+	strncpy(reinterpret_cast<char*>(file.se_xattr), "", 0);
+#endif
 	
 	int fdfile;
 
@@ -116,7 +131,6 @@ void UnixFS::readDevice(const std::string &path, const struct stat &filestat) co
 	log->loopDebug("UnixFS::readDevice(path=>%s, filestat=>0x%x) start", path.c_str(), &filestat);
 	
 	Doclone::unixFile file = {};
-	security_context_t context;
 
 	std::string fileName = basename(path.c_str());
 	strncpy(reinterpret_cast<char*>(file.name), fileName.c_str(), 512);
@@ -128,13 +142,18 @@ void UnixFS::readDevice(const std::string &path, const struct stat &filestat) co
 	file.mtime = filestat.st_mtime;
 	file.dev = filestat.st_rdev;
 	
+#ifdef HAVE_SELINUX
 	// Selinux extended attributes
+	security_context_t context;
 	if(getfilecon (path.c_str(), &context)<0) {
 		strncpy(reinterpret_cast<char*>(file.se_xattr), "", 0);
 	}
 	else {
 		strncpy (reinterpret_cast<char*>(file.se_xattr), context, 128);
 	}
+#else
+	strncpy(reinterpret_cast<char*>(file.se_xattr), "", 0);
+#endif
 
 	this->writeFileHeader(file);
 	
@@ -156,7 +175,6 @@ void UnixFS::readSymlink(const std::string &path, const struct stat &filestat) c
 	
 	char linkPath[4096]  = {};
 	Doclone::unixFile file  = {};
-	security_context_t context;
 	int size = 0;
 	
 	// Read link
@@ -173,12 +191,18 @@ void UnixFS::readSymlink(const std::string &path, const struct stat &filestat) c
 	file.gid = filestat.st_gid;
 	file.atime = filestat.st_atime;
 	file.mtime = filestat.st_mtime;
+
+#ifdef HAVE_SELINUX
+	security_context_t context;
 	if(lgetfilecon (path.c_str(), &context)<0) {
 		strncpy(reinterpret_cast<char*>(file.se_xattr), "", 0);
 	}
 	else {
 		strncpy (reinterpret_cast<char*>(file.se_xattr), context, 128);
 	}
+#else
+	strncpy(reinterpret_cast<char*>(file.se_xattr), "", 0);
+#endif
 
 	this->writeFileHeader(file);
 
@@ -203,7 +227,6 @@ void UnixFS::readHardLink(const std::string &path, const struct stat &filestat)
 	log->loopDebug("UnixFS::readHardLink(path=>%s, filestat=>0x%x) start", path.c_str(), &filestat);
 	
 	Doclone::unixFile file = {};
-	security_context_t context;
 	std::map<ino_t, std::string>::iterator it;
 	DataTransfer *trns = DataTransfer::getInstance();
 
@@ -216,13 +239,18 @@ void UnixFS::readHardLink(const std::string &path, const struct stat &filestat)
 	file.atime = filestat.st_atime;
 	file.mtime = filestat.st_mtime;
 
+#ifdef HAVE_SELINUX
 	// Selinux extended attributes
+	security_context_t context;
 	if(getfilecon (path.c_str(), &context)<0) {
 		strncpy(reinterpret_cast<char*>(file.se_xattr), "", 0);
 	}
 	else {
 		strncpy (reinterpret_cast<char*>(file.se_xattr), context, 128);
 	}
+#else
+	strncpy(reinterpret_cast<char*>(file.se_xattr), "", 0);
+#endif
 	
 	it = this->_hardLinks.find(filestat.st_ino);
 	
@@ -270,7 +298,6 @@ void UnixFS::readPipe(const std::string &path, const struct stat &filestat) cons
 	log->loopDebug("UnixFS::readPipe(path=>%s, filestat=>0x%x) start", path.c_str(), &filestat);
 	
 	Doclone::unixFile file = {};
-	security_context_t context;
 
 	std::string fileName = basename(path.c_str());
 	strncpy(reinterpret_cast<char*>(file.name), fileName.c_str(), 512);
@@ -281,13 +308,18 @@ void UnixFS::readPipe(const std::string &path, const struct stat &filestat) cons
 	file.atime = filestat.st_atime;
 	file.mtime = filestat.st_mtime;
 	
+#ifdef HAVE_SELINUX
 	// Selinux extended attributes
+	security_context_t context;
 	if(getfilecon (path.c_str(), &context)<0) {
 		strncpy(reinterpret_cast<char*>(file.se_xattr), "", 0);
 	}
 	else {
 		strncpy (reinterpret_cast<char*>(file.se_xattr), context, 128);
 	}
+#else
+	strncpy(reinterpret_cast<char*>(file.se_xattr), "", 0);
+#endif
 
 	this->writeFileHeader(file);
 	
@@ -332,7 +364,6 @@ void UnixFS::readDir(const std::string &path) throw(Exception) {
 			case S_IFDIR: {
 				if (strcmp (".", d_file->d_name) && strcmp ("..", d_file->d_name)) {
 					Doclone::unixFile file = {};
-					security_context_t context;
 
 					// write the dir in fd
 					strncpy (reinterpret_cast<char*>(file.name),
@@ -343,21 +374,27 @@ void UnixFS::readDir(const std::string &path) throw(Exception) {
 					file.size = 0;
 					file.atime = filestat.st_atime;
 					file.mtime = filestat.st_mtime;
+#ifdef HAVE_SELINUX
 					// Selinux extended attributes
-					if(getfilecon (abPath.c_str(), &context)<0) {
-						strncpy(reinterpret_cast<char*>(file.se_xattr),
-								"", 0);
-					} else {
-						strncpy (reinterpret_cast<char*>(file.se_xattr),
-								context, 128);
+					security_context_t context;
+					if(getfilecon (path.c_str(), &context)<0) {
+						strncpy(reinterpret_cast<char*>(file.se_xattr), "", 0);
 					}
+					else {
+						strncpy (reinterpret_cast<char*>(file.se_xattr), context, 128);
+					}
+#else
+					strncpy(reinterpret_cast<char*>(file.se_xattr), "", 0);
+#endif
 
 					this->writeFileHeader(file);
 
 					/*
-					 * If the current folder is a virtual one, bypass
+					 * If the current folder is a virtual one or is the mount
+					 * point of another partition, bypass it.
 					 */
-					if(Util::isVirtualDirectory(abPath.c_str())) {
+					if(Util::isVirtualDirectory(abPath.c_str())
+						|| Util::isMountPoint(abPath)) {
 						/*
 						 * Write a empty unixFile object that indicates the end
 						 * of the virtual folder
@@ -455,8 +492,10 @@ void UnixFS::writeRegularFile(const std::string &path, Doclone::unixFile &file)
 	chown (path.c_str(), file.uid, file.gid);
 	utime (path.c_str(), &time);
 	chmod (path.c_str(), file.mode);
-	
+
+#ifdef HAVE_SELINUX
 	setfilecon (path.c_str(), reinterpret_cast<char*>(file.se_xattr));
+#endif
 
 	/*
 	 * If this file cannot be transferred for any reason, we need its
@@ -508,8 +547,9 @@ void UnixFS::writeDevice(const std::string &path, Doclone::unixFile &file) const
 	chown (path.c_str(), file.uid, file.gid);
 	utime (path.c_str(), &time);
 	
+#ifdef HAVE_SELINUX
 	setfilecon (path.c_str(), reinterpret_cast<char*>(file.se_xattr));
-	
+#endif
 	log->loopDebug("UnixFS::writeDevice() end");
 }
 
@@ -543,8 +583,9 @@ void UnixFS::writeSymlink(const std::string &path, Doclone::unixFile &file) cons
 	
 	lchown (path.c_str(), file.uid, file.gid);
 	
+#ifdef HAVE_SELINUX
 	lsetfilecon (path.c_str(), reinterpret_cast<char*>(file.se_xattr));
-
+#endif
 	delete[] linkPath;
 	
 	log->loopDebug("UnixFS::writeSymlink() end");
@@ -594,8 +635,9 @@ void UnixFS::writeHardLink(const std::string &path, Doclone::unixFile &file) con
 		throw ex;
 	}
 	
+#ifdef HAVE_SELINUX
 	lsetfilecon (path.c_str(), reinterpret_cast<char*>(file.se_xattr));
-	
+#endif
 	log->loopDebug("UnixFS::writeHardLink() end");
 }
 
@@ -621,8 +663,9 @@ void UnixFS::writePipe(const std::string &path, Doclone::unixFile &file) const t
 	chown (path.c_str(), file.uid, file.gid);
 	utime (path.c_str(), &time);
 	
+#ifdef HAVE_SELINUX
 	setfilecon (path.c_str(), reinterpret_cast<char*>(file.se_xattr));
-	
+#endif
 	log->loopDebug("UnixFS::writePipe() end");
 }
 
@@ -661,8 +704,9 @@ void UnixFS::writeDir(const std::string &path) throw(Exception) {
 				utime (abPath.c_str(), &time);
 				chmod (abPath.c_str(), file.mode);
 
-				setfilecon (abPath.c_str(), reinterpret_cast<char*>(file.se_xattr));
-
+#ifdef HAVE_SELINUX
+				setfilecon (path.c_str(), reinterpret_cast<char*>(file.se_xattr));
+#endif
 				abPath.append("/");
 				this->writeDir (abPath);	// enter the directory
 				break;
