@@ -1,6 +1,6 @@
 /*
  *  libdoclone - library for cloning GNU/Linux systems
- *  Copyright (C) 2013 Joan Lledó <joanlluislledo@gmail.com>
+ *  Copyright (C) 2013, 2014 Joan Lledó <joanlluislledo@gmail.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,9 +17,26 @@
  */
 
 #include <doclone/Util.h>
+
+#include <stdlib.h>
+#include <stdio.h>
+#include <mntent.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <sys/wait.h>
+#include <unistd.h>
+#include <signal.h>
+#include <string.h>
+#include <regex.h>
+
+#include <sstream>
+#include <string>
+#include <fstream>
+
+#include <blkid/blkid.h>
+
 #include <doclone/Logger.h>
 #include <doclone/Clone.h>
-
 #include <doclone/exception/Exception.h>
 #include <doclone/exception/WriteDataException.h>
 #include <doclone/exception/NoAccessToDeviceException.h>
@@ -32,20 +49,7 @@
 #include <doclone/exception/SignalCaughtException.h>
 #include <doclone/exception/NoDeviceDriverRecognizedException.h>
 #include <doclone/exception/SigAbrtException.h>
-
-#include <stdlib.h>
-#include <stdio.h>
-#include <mntent.h>
-#include <sys/stat.h>
-#include <unistd.h>
-#include <signal.h>
-#include <string.h>
-#include <regex.h>
-#include <blkid/blkid.h>
-
-#include <sstream>
-#include <string>
-#include <fstream>
+#include <doclone/exception/SpawnProcessException.h>
 
 namespace Doclone {
 
@@ -526,7 +530,7 @@ bool Util::match(const std::string &str, const std::string &regEx) {
 	log->debug("Util::match(str=>%s, regEx=>%s) start", str.c_str(), regEx.c_str());
 	bool retVal = false;
 
-	regex_t preg ;
+	regex_t preg;
 	int nmatch = 1 ;
 	regmatch_t pmatch[  nmatch ] ;
 	int rc = regcomp( &preg, regEx.c_str(), REG_EXTENDED | REG_ICASE | REG_NEWLINE ) ;
@@ -563,6 +567,42 @@ std::string Util::intToString(int num) throw(Exception) {
 	return retVal;
 }
 
+/**
+ * \brief Checks whether the passed folder is a mount point
+ *
+ * \param path
+ * 		The path of the folder
+ */
+bool Util::isMountPoint(const std::string &path) throw(Exception) {
+	Logger *log = Logger::getInstance();
+	log->debug("Util::isMountPoint(path=>%s) start", path.c_str());
+
+	bool retVal = false;
+
+	FILE *fp;
+	struct mntent *tmp;
+
+	fp = setmntent("/etc/mtab","r");
+
+	if(fp == 0) {
+		FileNotFoundException ex("/etc/mtab");
+		ex.logMsg();
+	}
+
+	// Read all the mount points of the system
+	while( (tmp = getmntent(fp)) != 0) {
+		// if it matches, return true
+		if(path.compare(tmp->mnt_dir) == 0) {
+			retVal = true;
+		}
+	}
+
+	endmntent(fp);
+
+	log->debug("Util::isMountPoint(retVal=>%d) end", retVal);
+	return retVal;
+}
+
 /**
  * \brief Determines whether the given folder is virtual or not
  *
@@ -573,7 +613,7 @@ std::string Util::intToString(int num) throw(Exception) {
  */
 bool Util::isVirtualDirectory(const char *path) {
 	Logger *log = Logger::getInstance();
-	log->debug("Util::isVirtualDirectory(path=>%s) start", path);
+	log->loopDebug("Util::isVirtualDirectory(path=>%s) start", path);
 
 	bool retVal = !(strcmp ("/dev", path)
 			&& strcmp ("/proc",path)
@@ -584,7 +624,7 @@ bool Util::isVirtualDirectory(const char *path) {
 			&& strcmp ("/sys",path)
 			&& strcmp ("/run",path));
 
-	log->debug("Util::isVirtualDirectory(retVal=>%d) end", retVal);
+	log->loopDebug("Util::isVirtualDirectory(retVal=>%d) end", retVal);
 	return retVal;
 }
 
@@ -598,12 +638,14 @@ bool Util::isVirtualDirectory(const char *path) {
  */
 bool Util::isLiveFile(const char *path) {
 	Logger *log = Logger::getInstance();
-	log->debug("Util::isLiveFile(path=>%s) start", path);
+	log->loopDebug("Util::isLiveFile(path=>%s) start", path);
 
 	Clone *dcl = Clone::getInstance();
+
+	//The image that is being created must not be cloned.
 	bool retVal = !strcmp (dcl->getImage().c_str(), path);
 
-	log->debug("Util::isLiveFile(retVal=>%d) end", retVal);
+	log->loopDebug("Util::isLiveFile(retVal=>%d) end", retVal);
 	return retVal;
 }
 
@@ -629,7 +671,7 @@ bool Util::isUUIDRepeated(const char *uuid) throw(Exception) {
 	int blDev = blkid_probe_all(cache);
 	if(blDev != 0) {
 		log->debug("Util::isUUIDRepeated(retVal=>%d) end", retVal);
-		// Returning true to enforce the mounting/unmounting of the device, just in case.
+		// Returning true to enforce mounting/unmounting of the device, just in case.
 		return true;
 	}
 
@@ -643,6 +685,7 @@ bool Util::isUUIDRepeated(const char *uuid) throw(Exception) {
 		numDevRep++;
 	}
 	blkid_dev_iterate_end(iter);
+	blkid_put_cache(cache);
 
 	retVal = numDevRep > 1;
 	log->debug("Util::isUUIDRepeated(retVal=>%d) end", retVal);
@@ -741,4 +784,122 @@ void Util::signalHandler(int sig) throw(Exception) {
 	return;
 }
 
+/**
+ * \brief Splits a string into an vector of strings
+ *
+ * \param string
+ * 		String to be splitted
+ * \param delim
+ * 		Delimitator
+ * \param elems
+ * 		Address of the vector where the strings will be written
+ */
+void Util::split(const std::string &string, char delim, std::vector<std::string> &elems) {
+	Logger *log = Logger::getInstance();
+	log->debug("Util::split(string=>%s, delim=>%c) start", string.c_str(), delim);
+
+	std::stringstream ss(string);
+	std::string item;
+
+	while (std::getline(ss, item, delim)) {
+		elems.push_back(item);
+	}
+
+	log->debug("Util::split() end");
+	return;
+}
+
+/**
+ * \brief Finds out whether a program is available through the path.
+ *
+ *
+ * \param program
+ * 		name of the program
+ *
+ * \return The path of the program, or an empty string if not found
+ */
+std::string Util::find_program_in_path(const std::string &program) {
+	Logger *log = Logger::getInstance();
+	log->debug("Util::find_program_in_path(program=>%s) start", program.c_str());
+
+	std::string path = getenv("PATH");
+
+	std::vector<std::string> elems;
+	split(path, ':', elems);
+
+	std::string retVal = "";
+
+	for(unsigned int i=0; i< elems.size(); i++) {
+		std::string abPath = elems.at(i)+"/"+program;
+		if (access(abPath.c_str(), X_OK) == 0) {
+			retVal = abPath;
+		}
+	}
+
+	log->debug("Util::find_program_in_path(retVal=>%s) end", retVal.c_str());
+	return retVal;
+}
+
+/**
+ * \brief Spawns a new process
+ *
+ * \param command
+ * 		The command line to be executed
+ * \param exitValue
+ * 		Pointer to an integer where the exit status will be written
+ */
+void Util::spawn_command_line_sync(const std::string &command, int *exitValue, std::string *output) throw(Exception) {
+	Logger *log = Logger::getInstance();
+	log->debug("Util::spawn_command_line_sync(command=>%s) start", command.c_str());
+
+	pid_t pid;
+	int status;
+	int fds[2]; //pipe ends
+
+	//If output is not null, we must use a pipe for redirecting the child's output
+	if(output && pipe(fds)) {
+		SpawnProcessException ex(command);
+		throw ex;
+	}
+
+	pid = fork();
+	if (pid < 0) {
+		SpawnProcessException ex(command);
+		throw ex;
+	} else if (pid == 0) { //Child process
+		//Child's output must be connected to the write end of the pipe.
+		if(output) {
+			close(fds[0]);
+			dup2(fds[1], STDOUT_FILENO);
+		}
+
+		//Launch the new process
+		execlp("/bin/sh", "sh", "-c", command.c_str(), (void *)0);
+	} else if(waitpid(pid, &status, 0) != pid) {
+		SpawnProcessException ex(command);
+		throw ex;
+	}
+
+	if(WIFEXITED(status)) {
+		*exitValue = WEXITSTATUS(status);
+	} else {
+		SpawnProcessException ex(command);
+		throw ex;
+	}
+
+	//Save the output in the std::string
+	if(output) {
+		char buff[4096] = {};
+		int retVal = 0;
+
+		do {
+			retVal = read(fds[0], buff, 4096);
+			*output += buff;
+		} while(retVal == 4096);
+	}
+
+	log->debug("Util::spawn_command_line_sync() end");
+	return;
+}
+
 }
