@@ -1,6 +1,6 @@
 /*
  * doclone - a frontend for libdoclone
- * Copyright (C) 2013 Joan Lledó <joanlluislledo@gmail.com>
+ * Copyright (C) 2013, 2014 Joan Lledó <joanlluislledo@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,14 +16,16 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <ConsoleView/ConsoleView.h>
-#include <CursesView/CursesView.h>
-
 #include <config.h>
 
 #include <locale.h>
 #include <libintl.h>
 
+#include <ConsoleView/ConsoleView.h>
+#ifdef HAVE_NCURSES
+#include <CursesView/CursesView.h>
+#endif
+
 int main(int argc, char** argv) {
 	setlocale(LC_ALL, "");
 	textdomain(PACKAGE);
@@ -33,9 +35,13 @@ int main(int argc, char** argv) {
 		ConsoleView view;
 		view.initView(argc, argv);
 	} else {
+#ifdef HAVE_NCURSES
 		CursesView *curView = CursesView::getInstance();
-
 		curView->flowControl();
+#else
+		ConsoleView view;
+		view.initView(argc, argv);
+#endif
 	}
 
 	return 0;
